/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2015, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** DM           Daniel Mysliwitz                                              **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2015-08-07, DM:   Initial version                                  **
** V1.0.1: 2016-03-18, DM:   PORT added                                       **
*******************************************************************************/
 
/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** DMA: Scatter-gather                                                        **
********************************************************************************
** Four DMA tasks are defined. Each one copies data from one variable into    **
** another one.                                                               **
** DMA.Ch0 is set for Memory Scatter-Gater mode and operates on the 4 tasks.  **
** The DMA request for Ch0 is software-triggered                              **
*******************************************************************************/

#include "tle_device.h"

uint32 data0, data1;
uint16 data2, data3;
uint32 data4, data5;
uint32 data6, data7;

TDMA_Entry Task[DMA_CH0_TASK_NoOfTasks];

int main(void)
{
  /*****************************************************************************
  ** Initialization of the hardware modules based on the configuration set    **
  ** in Config Wizard                                                         **
  *****************************************************************************/
  TLE_Init();
  
	/* Task[0] copies data0 to data1 */
	data0 = 0x12345678;
	data1 = 0;
	
	/* Task[1] copies data2 to data3 */
	data2 = 0xABCD;
	data3 = 0;
	
	/* Task[2] copies data4 to data5 */
	data4 = 0xAA55CC33;
	data5 = 0;

	/* Task[3] copies data6 to data7 */
	data6 = 0xDEADBEEF;
	data7 = 0;
	
	DMA_Task_Set(&Task[0], DMA_Cycle_Type_MemSctGthAlt, 0, (uint32)&data0, (uint32)&data1, 1, DMA_32Bit_Transfer, DMA_No_Inc);
	DMA_Task_Set(&Task[1], DMA_Cycle_Type_MemSctGthAlt, 0, (uint32)&data2, (uint32)&data3, 1, DMA_16Bit_Transfer, DMA_No_Inc);
	DMA_Task_Set(&Task[2], DMA_Cycle_Type_MemSctGthAlt, 0, (uint32)&data4, (uint32)&data5, 1, DMA_32Bit_Transfer, DMA_No_Inc);
	DMA_Task_Set(&Task[3], DMA_Cycle_Type_Basic,        0, (uint32)&data6, (uint32)&data7, 1, DMA_32Bit_Transfer, DMA_No_Inc);

	/* Enable Dma master */
  DMA_Master_En();
	
	/* Set software request for DMA Channels */
  DMA_Software_Request_Set(DMA_MASK_CH0);

  for (;;)
  {
    (void)WDT1_Service();
  }
}

