/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2015, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** BG           Blandine Guillot                                              **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2020-02-27, BG:   Initial version of revision history              **
*******************************************************************************/
 
/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** DMA: Send out 4 SPI data words through SSC1 triggered by DMA               **
********************************************************************************
** The first SPI transfer (0x1234) is initiated manually.                     **
** Then, based on the transmit ready indication given by the SSC1 module,     **
** the DMA is triggered and fills in the next words into the SPI sendbuffer.  **
** To decode SPI message from SSC1, here are the following setting in Config  **
** Wizard:                                                                    **
** - SPI CLK : P0.3 (low when inactive)                                       **
** - SPI MOSI: P0.2                                                           **
** - SPI MISO: P0.4                                                           **
** - Least significant bit first                                              **
** - 16 bits per transfer                                                     **
** - Data valid on clock trailing edge                                        **
** - Enable line is active low                                                **
*******************************************************************************/

#include "tle_device.h"

uint16 spi_tx_data[DMA_CH2_NoOfTrans] = {0xAAAA, 0x3333, 0x0F0F, 0x00FF};

int main(void)
{
  /*****************************************************************************
  ** Initialization of the hardware modules based on the configuration set    **
  ** in Config Wizard                                                         **
  *****************************************************************************/
  TLE_Init();

  /* Enable the DMA master */
  DMA_Master_En();

  /* Send the first SPI data and start DMA along with it */
  SSC1_SendWord(0x1234);
  
  for (;;)
  {
    (void)WDT1_Service();
  }
}
