/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2015, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** BG           Blandine Guillot                                              **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2020-02-27, BG:   Initial version of revision history              **
*******************************************************************************/
 
/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** DMA: Send out a 'Hello World!' string at UART2 triggered by DMA            **
********************************************************************************
** In a terminal window set up a serial connection @115.2kBaud through the    **
** debugger port.                                                             **
** The Core timer 3 of GPT12E triggers a DMA transfer (DMA_CH12) every 100us. **
** The DMA takes one character out of the "uart_tx_data' string and puts it   **
** into the send buffer of UART2.                                             **
** The UART2 sends out the data at P1.1/P1.2 @115.2kBaud.                     **
*******************************************************************************/

#include "tle_device.h"
#include <stdio.h>

char uart_tx_data[DMA_CH12_NoOfTrans] = "Hello World!\r\n";

int main(void)
{
  /*****************************************************************************
  ** Initialization of the hardware modules based on the configuration set    **
  ** in Config Wizard                                                         **
  *****************************************************************************/
  TLE_Init();

  /* Enable DMA master */
  DMA_Master_En();
  
  /* Start T3 of GPT12E to start DMA-triggered UART transfer */
  GPT12E_T3_Start();
  
  for (;;)
  {
    (void)WDT1_Service();

    /* Wait for MON button pressed */
    if (MON_Debounce(0u) == 0u)
    {
      /* Start T3 of GPT12E to start DMA-triggered UART transfer */
      GPT12E_T3_Start();
    }
  }
}

/* Callback function on DMA complete */
void DMA_UART_SEND_Complete(void)
{
  /* Stop T3 of GPT12E */
  GPT12E_T3_Stop();
  
  /* Sends a 'DMA done' through UART2 */
  printf("DMA done\n");
  
  /* Reload DMA Channel 12 */
  DMA_Reset_Channel(DMA_CH12, DMA_CH12_NoOfTrans);
}
