/* 
 ***********************************************************************************************************************
 *
 * Copyright (c) 2015, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** JM           Jahangir Mansoori                                             **
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2016-07-01, JM:   Initial version                                  **
** V1.0.1: 2020-11-05, JO:   Updated description of the example               **
*******************************************************************************/
 
/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** GPT12E: Input Capture                                                      **
********************************************************************************
** Timer2 generates a signal at P0.2 (callback TIMER2_ISR1 is called at       **
** Timer2 over-/ underflow every after every 26.66ms and toggles P2.0)        **
** GPT1T3 is set up as timer with a frequency of 187.5kHz, it toggles P0.0    **
** at every overflow (every 1/187.5kHz * 2^16 ~350ms) in the callback         **
** TIMER2_ISR1                                                                **
** GPT1T2 is configured to capture on rising and falling edge at T2IN (P1.4). **
** Use a jumper wire to connect P0.2 (signal generated by T2) with P1.4.      **
** At every capture event, the callback T2_CAP_ISR is called, calculates      **
** the ticks and stores the calcualted value in the variable final_counts.    **
** The value of final_counts shall be ca.                                     **
** 187.5kHz * 26.66ms = 4998 (~0x1386)                                        **
*******************************************************************************/

#include "tle_device.h"

uint32 flag_overflow;
uint32 counts, previous_counts;
uint32 final_counts;

int main(void)
{
  /*****************************************************************************
  ** Initialization of the hardware modules based on the configuration set    **
  ** in Config Wizard                                                         **
  *****************************************************************************/
  TLE_Init();
  
  TIMER2_Start();
  GPT12E_T3_Start();

  for (;;)
  {
    (void)WDT1_Service();
  }
}


/* Timer2 toggles P0.2 for each overflow after 26.66ms */
void TIMER2_ISR1(void)
{
  PORT_ChangePin(0x02, PORT_ACTION_TOGGLE);
}


/* Capture ISR, Rising-Falling interrupt at P1.4 */
void T2_CAP_ISR(void)
{
  counts = GPT12E_T2_Value_Get();
  if (flag_overflow == 0)
  {
    final_counts = counts - previous_counts;
  }
  else
  {
    final_counts = (uint32)((0xffff - previous_counts) + counts);
    flag_overflow--;
  }
  previous_counts = counts;
}


/* Core timer 3 of GPT12E toggles P0.0 for each overflow */
void T3_ISR_Overflow(void)
{
  flag_overflow++;
  
  if (flag_overflow > 2)
  {
    flag_overflow = 0;
    counts = 0;
    final_counts = 0;
  }
}
