/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2015, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** BG           Blandine Guillot                                              **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.0.0: 2020-02-27, BG:   Initial version of revision history              **
*******************************************************************************/
 
/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** PMU: VDDEXT enable and diagnostic function                                 **
********************************************************************************
** VDDEXT is enabled (5V at VDDEXT pin)                                       **
** LED1 will be set upon VDDEXT short to GND                                  **
** LED2 will be set upon overvoltage at VDDEXT                                **
** LED3 will be set upon overload at VDDEXT                                   **
** High Level on P1.0 clears LEDs and switches on VDDEXT again                **
*******************************************************************************/

#include "tle_device.h"
#include "eval_board.h"
#include "scu.h"

static uint8 bVDDEXT_Fail;

int main(void)
{
  /*****************************************************************************
  ** Initialization of the hardware modules based on the configuration set    **
  ** in Config Wizard                                                         **
  *****************************************************************************/
  TLE_Init();

  /* Disable VAREF, as VAREF is connected to VDDEXT on the evalboard */
  MF->REF2_CTRL.bit.VREF5V_PD_N = 0u;

  /* Initialize VDDEXT fail indication */
  bVDDEXT_Fail = 0;

  for (;;)
  {
    (void)WDT1_Service();

    /* Check if VDDEXT has failed */
    if (bVDDEXT_Fail == 1u)
    {
      /* Check if clear request for VDDEXT is active */
      if (PORT_ReadPin(0x10) == 1u)
      {
        /* Reset error LEDs */
        PORT_ChangePin(LED1, PORT_ACTION_CLEAR);
        PORT_ChangePin(LED2, PORT_ACTION_CLEAR);
        PORT_ChangePin(LED3, PORT_ACTION_CLEAR);
        /* Clear VDDEXT error flags */
        PMU_VDDEXT_Short_Clr();
        /* Reset VDDEXT fail indication variable */
        bVDDEXT_Fail = 0u;
      }
    }
  }
}


/* Callback function on short diagnosis */
void VDDEXT_Short_Diag_Callback(void)
{
  /* Apply a delay of approx. 150us to give error flags in VDDEXT_CTRL time to show up */
  Delay_us(150);
  /* Set error LED1 */
  PORT_ChangePin(LED1, PORT_ACTION_SET);
  /* Set VDDEXT fail indication variable */
  bVDDEXT_Fail = 1u;
}


/* Callback function on overvoltage diagnosis */
void VDDEXT_Over_Volt_Diag_Callback(void)
{
  /* Apply a delay of approx. 150us to give error flags in VDDEXT_CTRL time to show up */
  Delay_us(150);
  /* Set error LED2 */
  PORT_ChangePin(LED2, PORT_ACTION_SET);
  /* Set VDDEXT fail indication variable */
  bVDDEXT_Fail = 1u;
}


/* Callback function on overload diagnosis */
void VDDEXT_Over_Load_Diag_Callback(void)
{
  /* Apply a delay of approx. 150us to give error flags in VDDEXT_CTRL time to show up */
  Delay_us(150);
  /* Set error LED3 */
  PORT_ChangePin(LED3, PORT_ACTION_SET);
  /* Set VDDEXT fail indication variable */
  bVDDEXT_Fail = 1u;
}
