/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2015, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V1.3.0: 2020-02-28, JO:   Initial version of the example, published in     **
**                           TLE986x Pack 1.4.4                               **
*******************************************************************************/
 
/*******************************************************************************
**                                  Abstract                                  **
********************************************************************************
** Timer3: Mode1B - 16-bit timer triggered by an event                        **
********************************************************************************
** CCU6 Ch0 generates a pwm signal at P0.4 with a frequency of 10kHz and a    **
** duty cycle of 50%.                                                         **
**                                                                            **
** At every rising edge of the pwm signal, Timer3 is reset and                **
** starts counting.                                                           **
** After 10us, Timer3 triggers an ADC1 measurement and an interrupt.          **
** To visualize the interrupt, P1.1 is set to high in the interrupt.          **
** When the ADC1 measurement has finished, an ADC1 EOC interrupt is           **
** generated ('Poti_Handler') which sets P1.1 to low and visualizes the       **
** measured value on the LEDs.                                                **
**                                                                            **
********************************************************************************
**           ____________________                    ___________________      **
** P0.4     |                    |                  |                   ...   **
** CC60 ----                      ------------------                          **
**          : reset & start                         : reset & start           **
**          : Timer3 (10us)                         : Timer3 (10us)           **
**          :         _                             :         _               **
** ADC1 Trig:        | |                            :        | |              **
**      ----:--------:  ----------------------------:--------:  ---------     **
**          :<------>:                              :<------>:                **
**             10us  :                                 10us  :                **
** ADC1 Meas         :<------>:                              :<------>:       **
**                   : 1.4us  :                              : 1.4us  :       **
**                   :        :_                             :        :_      **
** ADC1 EOC          :        | |                            :        | |     **
**      -------------:--------:  ----------------------------:--------:  -    **
**                   :________:                              :________:       **
** P1.1              |        |                              |        |       **
**      -------------          ------------------------------          -...   **
** Toggling of P1.1 is delayed due to interrupt latency (enter interrupt,     **
** return from interrupt), ISR handling, and Port toggling                    **
*******************************************************************************/

#include "tle_device.h"

int main(void)
{
  /*****************************************************************************
  ** Initialization of the hardware modules based on the configuration set    **
  ** in Config Wizard                                                         **
  *****************************************************************************/
  TLE_Init();
  
  /* Start Timer3 */
  TIMER3_Start();
  /* Start PWM generation */
	CCU6_T12_Start();
	
  for (;;)
  {
    (void)WDT1_Service();
  }
}


/* Callback function for each overflow of Timer3 */
void Timer3_HB_Ovf_INT(void)
{
	PORT_P11_Output_High_Set();
}


void Poti_Handler(void)
{
#define Interval_mV 625u
  uint16 mV;
  bool value_valid;
	
	PORT_P11_Output_Low_Set();
  	
  value_valid = ADC1_GetChResult_mV(&mV, ADC1_CH4);

  if (value_valid == true)
  {
    if (mV <= (1u * Interval_mV))
    {
      PORT_P01_Output_High_Set();
			PORT_P01_Output_Low_Set();
    }
    if (mV > (1u * Interval_mV))
    {
      PORT_P01_Output_High_Set();
      PORT_P02_Output_Low_Set();
    }
    if (mV > (2u * Interval_mV))
    {
      PORT_P02_Output_High_Set();
      PORT_P03_Output_Low_Set();
    }
    if (mV > (3u * Interval_mV))
    {
      PORT_P03_Output_High_Set();
      PORT_P12_Output_Low_Set();
    }
    if (mV > (4u * Interval_mV))
    {
      PORT_P12_Output_High_Set();
      PORT_P10_Output_Low_Set();
    }
    if (mV > (5u * Interval_mV))
    {
      PORT_P10_Output_High_Set();
      PORT_P13_Output_Low_Set();
    }
    if (mV > (6u * Interval_mV))
    {
      PORT_P13_Output_High_Set();
      PORT_P14_Output_Low_Set();
    }
    if (mV > (7u * Interval_mV))
    {
      PORT_P14_Output_High_Set();
    }
  }
}

