/*
 ***********************************************************************************************************************
 *
 * Copyright (c) 2015, Infineon Technologies AG
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,are permitted provided that the
 * following conditions are met:
 *
 *   Redistributions of source code must retain the above copyright notice, this list of conditions and the  following
 *   disclaimer.
 *
 *   Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *   following disclaimer in the documentation and/or other materials provided with the distribution.
 *
 *   Neither the name of the copyright holders nor the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT  OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 **********************************************************************************************************************/
/**
 * \file     tle_variants.h
 *
 * \brief    Device specific memory layout defines
 *
 * \version  V0.2.0
 * \date     12. Oct 2020
 */

/*******************************************************************************
**                             Author(s) Identity                             **
********************************************************************************
**                                                                            **
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** DM           Daniel Mysliwitz                                              **
** JO           Julia Ott                                                     **
*******************************************************************************/

/*******************************************************************************
**                          Revision Control History                          **
********************************************************************************
** V0.1.0: 2019-01-29, DM:   initial version                                  **
** V0.1.1: 2019-02-23, DM:   sNADStart defined to be ARMCC v6 compliant       **
** V0.1.2: 2019-08-01, JO:   Added UC_FEATURE_* defines                       **
** V0.1.3: 2019-10-21, JO:   Added UC_FEATURE_STOPMODE define                 **
** V0.1.4: 2020-10-08, JO:   EP-507: Updated revision history format          **
** V0.2.0: 2020-10-12, JO:   EP-503: Added support for TLE9862QXA40           **
*******************************************************************************/

#ifndef _TLE_VARIANTS_H
#define _TLE_VARIANTS_H

/* Family definitions  */
#define TLE98      (98)

/* Series definitions  */
#define TLE984     (984)
#define TLE985     (985)
#define TLE986     (986)
#define TLE987     (987)

/* Device definitions  */
#define TLE9861    (9861)
#define TLE9867    (9867)
#define TLE9869    (9869)

/* Package definitions */
#define BGA144     (1)
#define LQFP144    (2)
#define LQFP100    (3)
#define BGA64      (4)
#define LQFP64     (5)
#define VQFN48     (6)
#define TSSOP38    (7)
#define TSSOP28    (8)
#define TSSOP16    (9)
#define VQFN24     (10)
#define VQFN40     (11)


#if defined TLE9861QXA20
  #define UC_FAMILY       TLE98
  #define UC_SERIES       TLE986
  #define UC_DEVICE       TLE9861
  #define UC_PACKAGE      VQFN48
  #define UC_FLASH        (36UL)
  #define UC_FEATURE_DMA
  #define UC_FEATURE_STOPMODE
  #define ProgFlashSize   (0x8000U)
  #define RAMSize         (0xC00U)
  #define sNADStart       ".ARM.__at_0x11007FFC"

#elif defined TLE9867QXA20
  #define UC_FAMILY       TLE98
  #define UC_SERIES       TLE986
  #define UC_DEVICE       TLE9867
  #define UC_PACKAGE      VQFN48
  #define UC_FLASH        (64UL)
  #define UC_FEATURE_DMA
  #define UC_FEATURE_STOPMODE
  #define ProgFlashSize   (0xF000U)
  #define RAMSize         (0x1800U)
  #define sNADStart       ".ARM.__at_0x1100EFFC"

#elif defined TLE9867QXW20
  #define UC_FAMILY       TLE98
  #define UC_SERIES       TLE986
  #define UC_DEVICE       TLE9867
  #define UC_PACKAGE      VQFN48
  #define UC_FLASH        (64UL)
  #define UC_FEATURE_DMA
  #define UC_FEATURE_STOPMODE
  #define ProgFlashSize   (0xF000U)
  #define RAMSize         (0x1800U)
  #define sNADStart       ".ARM.__at_0x1100EFFC"

#elif defined TLE9867QXA40
  #define UC_FAMILY       TLE98
  #define UC_SERIES       TLE986
  #define UC_DEVICE       TLE9867
  #define UC_PACKAGE      VQFN48
  #define UC_FLASH        (64UL)
  #define UC_FEATURE_DMA
  #define UC_FEATURE_STOPMODE
  #define ProgFlashSize   (0xF000U)
  #define RAMSize         (0x1800U)
  #define sNADStart       ".ARM.__at_0x1100EFFC"

#elif defined TLE9868QXB20
  #define UC_FAMILY       TLE98
  #define UC_SERIES       TLE986
  #define UC_DEVICE       TLE9868
  #define UC_PACKAGE      VQFN48
  #define UC_FLASH        (128UL)
  #define UC_FEATURE_SDADC
  #define ProgFlashSize   (0x1F000U)
  #define RAMSize         (0x1000U)
  #define sNADStart       ".ARM.__at_0x1101EFFC"

#elif defined TLE9869QXA20
  #define UC_FAMILY       TLE98
  #define UC_SERIES       TLE986
  #define UC_DEVICE       TLE9869
  #define UC_PACKAGE      VQFN48
  #define UC_FLASH        (128UL)
  #define UC_FEATURE_DMA
  #define UC_FEATURE_STOPMODE
  #define ProgFlashSize   (0x1F000U)
  #define RAMSize         (0x1800U)
  #define sNADStart       ".ARM.__at_0x1101EFFC"

#elif defined TLE9862QXA40
  #define UC_FAMILY       TLE98
  #define UC_SERIES       TLE986
  #define UC_DEVICE       TLE9862
  #define UC_PACKAGE      VQFN48
  #define UC_FLASH        (256UL)
  #define UC_FEATURE_DMA
  #define UC_FEATURE_STOPMODE
  #define ProgFlashSize   (0x3F000U)
  #define RAMSize         (0x2000U)
  #define sNADStart       ".ARM.__at_0x1103EFFC"

#else
  #error "tle_device.h: device not supported"
#endif

#if defined UC_FEATURE_DMA && defined UC_FEATURE_SDADC
  #define UC_FEATURE_DMA_CH13
#endif /* defined UC_FEATURE_DMA && defined UC_FEATURE_SDADC */
/*----------------------------------------------------------------------------
  Define Memory
 *----------------------------------------------------------------------------*/
#define ProgFlashStart  (0x11000000U)      /* Start Address of the flash      */
#define DataFlashStart  (ProgFlashStart + ProgFlashSize)
#define DataFlashSize   (0x1000U)          /* 4KB Data Flash                  */
#define NACStart        (DataFlashStart - 4U)
#define NADStart        (DataFlashStart - 2U)
#define RAMStart        (0x18000000UL)     /* Start Address of the SRAM       */

#if (UC_SERIES == 986)
  #include "tle986x.h"
#endif

#endif
